/**
 * Copyright 2015, Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jbpm.examples.util;

import javax.enterprise.event.Observes;
import javax.enterprise.inject.spi.BeforeBeanDiscovery;
import javax.enterprise.inject.spi.Extension;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.Name;
import javax.naming.NameNotFoundException;
import javax.naming.NamingException;


public class ConfigureBean implements Extension {

    
	void beforeBeanDiscovery(@Observes BeforeBeanDiscovery bbd) {
        	
    	bindToJNDI("java:jboss/FakeUT", new FakeUserTransaction());
    	
    	System.setProperty("jbpm.ut.jndi.lookup", "java:jboss/FakeUT");
    }
    
    protected void bindToJNDI(String businessKey, Object toBind) {
        try {
            Context ctx = new InitialContext();
            Name name = ctx.getNameParser("").parse(businessKey);
            
            int size = name.size();
            String atom = name.get(size - 1);
            Context parentCtx = createSubcontext(ctx, name.getPrefix(size - 1));
            parentCtx.bind(atom, toBind);
            
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected Context createSubcontext(Context ctx, Name name) throws NamingException {
        Context subctx = ctx;
        for (int pos = 0; pos < name.size(); pos++) {
            String ctxName = name.get(pos);
            try {
                subctx = (Context) ctx.lookup(ctxName);
            } catch (NameNotFoundException e) {
                subctx = ctx.createSubcontext(ctxName);
            }
            ctx = subctx;
        }
        return subctx;
    }
}
